from functools import cache
from typing import Mapping

from datasets import load_dataset

from evoprompt.helpers.prompts import BasePromptsFromJsonMixin
from evoprompt.task import TextClassification
from evoprompt.task.task import DatasetDatum


class Subj(BasePromptsFromJsonMixin, TextClassification):
    shorthand = "subj"
    base_prompts_file = "evoprompt/initial_prompts/subj/prompts.json"

    def load_validation_set(
        self, validation_dataset: str | None, validation_split: str | None
    ):
        return load_dataset(
            "csv",
            data_files={"validation": "evoprompt/data/subj/train.csv"},
            split="validation",
            column_names=["label", "text"],
        )

    def load_test_set(self, test_dataset: str, test_split: str | None):
        # this dataset is not in correct format therefore we process it after reading as a simple text file
        return load_dataset(
            "csv",
            data_files={"test": "evoprompt/data/subj/test.csv"},
            split="test",
            column_names=["label", "text"],
        )

    def _get_text_for_datum(self, datum: DatasetDatum) -> str:
        return datum["text"]

    def _get_gold_label_for_datum(self, datum: DatasetDatum) -> str:
        return datum["label"]

    @staticmethod
    def _get_generation_prefix():
        return "Subjectivity: "

    @staticmethod
    @cache
    def _get_label_mapping() -> Mapping:
        classes = ["subjective", "objective"]
        return dict(zip(classes, range(len(classes))))
